#!/usr/bin/python3
import random
import lorem
from dateutil.rrule import *
from datetime import *
import pendulum
import sys, os

def parse(s, **kwd):
    return pendulum.parse(s, strict=False, **kwd)

def phrase(minlen=24):
    # drop the ending period
    s = lorem.sentence()[:-1]
    tmp = ""
    words = s.split(' ')
    while words and len(tmp) < minlen:
        tmp += f"{words.pop(0)} "

    return tmp.strip()

def make_examples(egfile=None):
    num_items = 300
    num_konnections = 0
    # include 3 months - the previous, current and next months
    start = parse('9a 1') - pendulum.duration(months=1)
    until = parse('9a 1') + pendulum.duration(months=2) - pendulum.duration(days=1)
    now = parse('9a') - pendulum.duration(days=7)

    datetimes = list(rrule(DAILY, byweekday=range(7), byhour=range(6, 22), dtstart=start, until=until))

    types = ['-', '*', '%', '-']
    # clients = ['A', 'B', 'C', 'D', 'E']
    clients = ['A', 'B', 'C']
    projects = {
            'A': ['project a1', 'project a2'],
            'B': ['project b1', 'project b2', 'project b3'],
            'C': ['project c1', 'project c2'],
            'D': ['project d1'],
            'E': ['project e1', 'project e2', 'project e3'],
            }
    activities = {
            'A': ['activity a1', 'activity a2'],
            'B': ['activity b1', 'activity b2', 'activity b3'],
            'C': ['activity c1', 'activity c2'],
            'D': ['activity d1'],
            'E': ['activity e1', 'activity e2', 'activity e3'],
            }
    locations = ['errands', 'home', 'office', 'shop']
    tags = ['red', 'green', 'blue']
    dates = [0, 0, 1, 0, 0] # dates 1/5 of the time
    minutes = range(12, 96, 6) # for used times
    days = range(7)
    extent = range(60, 180, 30)

    client_contacts = {}
    client_id = {}
    examples = []

    examples.append("! the lorem examples @t lorem @d 1) This inbox item and each of the other internally generated reminders is tagged 'lorem'. All of them can be removed in one step by opening query view (press 'q'), entering the query 'any t lorem | remove' and pressing 'return'. 2). The examples are generated to fit within a three month period including the month they were generated together with the previous and subsequent months. You can remove and regenerate them whenever you like to keep them current.")

    # for client in clients:
    #     # client records
    #     num_contacts = random.randint(1, 4)
    #     client_contacts[client] = contact_ids
    #     # add links from client to contacts
    #     for i in range(len(contact_ids)):
    #         doc_id += 1
    #         # add contact records
    #         examples.append(f"% contact {client}{i+1} @i contact/client {client} @d {lorem.sentence()[:-1]} @t lorem")
    #     doc_id += 1
    #     client_id[client] = doc_id
    #     if num_konnections:
    #         # add clients with links from client to contacts
    #         tmp = ' '.join([f'@k {x}' for x in contact_ids])
    #     else:
    #         tmp = ''
    #     examples.append(f"% client {client} @i clients @d {lorem.sentence()[:-1]} {tmp} @t lorem")

    # konnections = []
    # if num_konnections:
    #     for _ in range(num_konnections):
    #         client = random.choice(clients)
    #         num_contacts = random.randint(1, len(client_contacts[client]))
    #         # examples.append(client_contacts, num_contacts)
    #         contacts = random.sample(client_contacts[client], k=num_contacts)
    #         contacts.sort()
    #         tmp = ' '.join([f'@k {x}' for x in contacts])

    #         konnections.append(f"@k {client_id[client]} {tmp}")

    for _ in range(num_items):
        t = random.choice(types)
        summary = phrase()
        start = random.choice(datetimes)
        date = random.choice(dates)
        s = start.strftime("%Y-%m-%d") if date else start.strftime("%Y-%m-%d %I:%M%p")
        d = lorem.paragraph()
        i1 = random.choice(clients)
        i2 = random.choice(projects[i1])
        i3 = random.choice(activities[i1])
        begin = random.choice(range(1, 15))
        used = ""
        # konnect = random.choice(konnections) if konnections and random.randint(1, 10) <= 4 else ""
        for i in range(random.randint(1,2)):
            u = random.choice(minutes)
            if random.choice(dates):
                e = start.strftime("%Y-%m-%d")
            else:
                e = (start + pendulum.duration(minutes=u)).strftime("%Y-%m-%d %I:%M%p")
            used += f"@u {u}m: {e} "

        if t == '*':
            if date:      # an event
                examples.append(f"{t} {summary} @s {s} @t {random.choice(tags)} @t lorem")
            else:
                x = random.choice(extent)
                examples.append(f"{t} {summary} @s {s} @e {x}m @i client {i1}/{i2}/{i3} {used} @d {d} @t {random.choice(tags)} @t lorem")
        elif t == '-' and random.choice(['h', 't']) == 'h':
            if start < now:
                examples.append(f"{t} {summary} @s {s} @i client {i1}/{i2}/{i3} @f {s} {used} @d {d} @t lorem")
            else:
                examples.append(f"{t} {summary} @s {s} @i client {i1}/{i2}/{i3} {used} @d {d} @b {begin} @t lorem")

        else:
            examples.append(f"{t} {summary} @i client {i1}/{i2}/{i3} {used} @d {d} @l {random.choice(locations)} @t {random.choice(tags)} @t lorem")

    if egfile:
        with open(egfile, 'w') as fo:
            fo.writelines("\n".join(examples))
    else:
        return examples

if __name__ == '__main__':
    if len(sys.argv) > 1:
        egfile = sys.argv.pop(1)
    else:
        egfile = None
    make_examples(egfile)




#! ./env/bin/python
def main():
    import sys
    import logging
    import logging.config
    logging.getLogger('asyncio').setLevel(logging.WARNING)
    logger = logging.getLogger()
    MIN_PYTHON = (3, 7, 3)
    if sys.version_info < MIN_PYTHON:
        mv = ".".join([str(x) for x in MIN_PYTHON])
        sys.exit(f"Python {mv} or later is required.\n")
    import os
    IS_VENV = os.getenv('VIRTUAL_ENV') is not None

    import etm.__version__ as version
    etm_version = version.version
    etmhome = os.environ.get("ETMHOME")
    etmdir = etmhome if etmhome else os.getcwd()

    loglevel = 2 # info
    log_levels = [str(x) for x in range(1, 6)]
    if len(sys.argv) > 1 and sys.argv[1] in log_levels:
        loglevel = int(sys.argv.pop(1))
    if len(sys.argv) > 1 and os.path.isdir(sys.argv[1]):
        etmdir = sys.argv.pop(1)

    logdir = os.path.normpath(os.path.join(etmdir, 'logs'))
    if not os.path.isdir(logdir):
        os.makedirs(logdir)

    backdir = os.path.normpath(os.path.join(etmdir, 'backups'))
    if not os.path.isdir(backdir):
        os.makedirs(backdir)

    import etm.options as options
    setup_logging = options.setup_logging
    setup_logging(loglevel, logdir)
    options.logger = logger
    Settings = options.Settings(etmdir)

    settings = Settings.settings
    beginbusy = settings['beginbusy']
    type_colors = settings['type_colors']
    # logger.debug(f"__main__ type_colors: {type_colors}")
    window_colors = settings['window_colors']
    # logger.debug(f"__main__ window_colors: {window_colors}")

    logger.info(f"running in a virtual environment: {IS_VENV}")

    secret = settings.get('secret')
    queries = settings.get('queries')
    import pendulum
    today = pendulum.today()
    # We want 2 char 'en' weekday abbreviations regardless of the actual locale
    day = today.end_of('week')  # Sunday
    WA = {i: day.add(days=i).format('ddd')[:2] for i in range(1, 8)}
    midnight = pendulum.now().replace(hour=0, minute=0, second=0, microsecond=0)
    beginbusy = settings.get('beginbusy', 7)
    ampm = settings.get('ampm', True)
    hour = pendulum.duration(hours=1)
    fmt = "hA" if ampm else "H"
    # busyhours = [(midnight + i*hour).format(fmt).lower for i in range(beginbusy, beginbusy + 14, 2)]
    busyhours = [(midnight + i*hour).format('hA').lower() for i in range(beginbusy, beginbusy + 15, 2)]
    HB = "".join([f"{h : <8}" for h in busyhours]).rstrip()
    # "".join([f"{h : <10}" for h in busyhours]).rstrip()

    # HSEP  =    '─' # U+2500  this will be a de-emphasized color
    VSEP   =    '⏐' # U+23D0  this will be a de-emphasized color
    HSEP   =    '·' # U+2500  this will be a de-emphasized color
    BUSY   =    '■' # U+25A0 this will be busy color
    CONF   =    '▦' # U+25A6 this will be conflict color


    import etm.ical as ical
    ical.logger = logger
    import etm.data as data
    data.secret = secret
    from etm.data import Mask
    dbfile = os.path.normpath(os.path.join(etmdir, 'db.json'))
    logger.debug(f"using dbfile: {dbfile}")
    cfgfile = os.path.normpath(os.path.join(etmdir, 'cfg.yaml'))
    ETMDB = data.initialize_tinydb(dbfile)
    DBITEM = ETMDB.table('items', cache_size=None)
    DBARCH = ETMDB.table('archive', cache_size=None)
    logger.debug(f"ETMDB: {ETMDB}, number of items: {len(ETMDB)}")
    # egfile = os.path.normpath(os.path.join(etmdir, 'examples.text'))
    # if not os.path.exists(egfile):
    #     logger.debug("egfile does not exist")
    #     from etm.make_examples import make_examples
    #     make_examples(egfile, len(ETMDB)+1, logger)

    from etm.make_examples import make_examples
    from etm.model import about
    from etm.model import wrap
    from etm.model import import_file
    from etm.model import import_examples
    from etm.model import write_back
    # from etm.model import RDict
    from etm.model import item_details
    from etm.model import FINISHED_CHAR
    from etm.model import UPDATE_CHAR
    from etm.model import PIN_CHAR
    from etm.model import INBASKET_CHAR
    import etm.model as model
    model.loglevel = loglevel
    model.etm_version = etm_version
    model.secret = secret
    model.data = data
    model.ical = ical
    model.Mask = Mask
    model.WA = WA
    model.HB = HB
    model.VSEP = VSEP
    model.HSEP = HSEP
    model.BUSY = BUSY
    model.CONF = CONF
    model.ETMDB = ETMDB
    model.DBITEM = DBITEM
    model.DBARCH = DBARCH
    model.beginbusy = beginbusy
    model.settings = settings
    model.logger = logger
    model.make_examples = make_examples
    # model.edit_file = os.path.join(etmdir, 'edit.text')
    model.timers_file = os.path.join(etmdir, 'timers.pkl')
    userhome = os.path.expanduser('~')
    etmhome = os.path.join('~', os.path.relpath(etmdir, userhome)) if etmdir.startswith(userhome) else etmdir
    model.etmhome = etmhome
    # we put settings into the model namespace so model.Dataview will have it
    dataview = model.DataView(etmdir)
    datetime_calculator = model.datetime_calculator
    item = model.Item()
    format_time = model.format_time
    format_datetime = model.format_datetime
    format_duration = model.format_duration
    format_hours_and_tenths = model.format_hours_and_tenths
    # since dataview calls schedule it will also have settings
    completions = dataview.completions
    expansions = settings["expansions"]
    if expansions:
        for x in expansions:
            completions.append(f"@x {x}")
    style = dataview.settings["style"]
    parse_datetime = model.parse_datetime
    parse_duration = model.parse_duration

    logger.info(f"initialized TinyDB using {dbfile}")


    import etm.view as view
    view.wrap = wrap
    view.WA = WA
    view.beginbusy = beginbusy
    view.FINISHED_CHAR = FINISHED_CHAR
    view.UPDATE_CHAR = UPDATE_CHAR
    view.PIN_CHAR = PIN_CHAR
    view.INBASKET_CHAR = INBASKET_CHAR
    view.VSEP = VSEP
    view.HSEP = HSEP
    view.BUSY = BUSY
    view.CONF = CONF
    view.settings = settings
    view.type_colors = type_colors
    view.cfgfile = cfgfile
    view.model = model
    view.write_back = write_back
    # view.RDict = RDict
    # view.TDBLexer = model.TBDLexer
    view.item = item
    view.item_details = item_details
    view.logger = logger
    view.import_file = import_file
    view.import_examples = import_examples
    view.etmdir = etmdir
    view.inbasket_file = os.path.join(etmdir, 'inbasket.text')
    view.etmhome = etmhome
    view.datetime_calculator = datetime_calculator
    view.about = about
    view.format_time = format_time
    view.format_datetime = format_datetime
    view.format_duration = format_duration
    view.parse_datetime = parse_datetime
    view.ETMDB = ETMDB
    view.DBITEM = DBITEM
    view.DBARCH = DBARCH
    view.etm_version = etm_version

    view.dataview = dataview
    view.completions = completions
    view.expansions = expansions
    view.terminal_style = style
    view.make_examples = make_examples
    from etm.view import ETMQuery

    import etm.report as report
    view.report = report
    show_query_results = report.show_query_results
    view.show_query_results = show_query_results
    model.show_query_results = show_query_results
    report.ETMDB = ETMDB
    report.DBITEM = DBITEM
    report.DBARCH = DBARCH
    report.ETMQuery = ETMQuery
    report.settings = settings
    report.format_time = format_time
    report.parse_duration = parse_duration
    report.parse_datetime = parse_datetime
    report.format_datetime = format_datetime
    report.format_duration = format_duration
    report.format_hours_and_tenths = format_hours_and_tenths
    report.logger = logger
    report.UT_MIN = settings.get('usedtime_minutes', 1)

    logger.info(f"setting terminal_style: {style}")


    if len(sys.argv) > 1:
        if sys.argv[1] == 'model':
            logger.info(f"calling model doctest with etmdir: {etmdir}, argv: {sys.argv}")
            import doctest
            doctest.testmod(model)
        elif sys.argv[1] == 'view':
            logger.info(f"calling view doctest with etmdir: {etmdir}, argv: {sys.argv}")
            import doctest
            doctest.testmod(view)
        elif sys.argv[1] == 'data':
            logger.info(f"calling data doctest with etmdir: {etmdir}, argv: {sys.argv}")
            import doctest
            doctest.testmod(data)
        elif sys.argv[1] == 'rep':
            logger.info(f"calling report.main with etmdir: {etmdir}, argv: {sys.argv}")
            report.main(etmdir, sys.argv)
        else:
            logger.info(f"calling model.main with etmdir: {etmdir}, argv: {sys.argv}")
            model.main(etmdir, sys.argv)

    else:
        logger.info(f"system info {model.about()[1]}")
        logger.info(f"calling view.main with etmdir: {etmdir}")
        from etm.view import main
        import asyncio
        asyncio.run(main(etmdir))

def inbasket():
    import sys
    import os
    typechar = '!' # inbasket
    option = '@t etm+'

    help = f"""\
usage: etm+ 'text'          use text
   or: etm+                 get text from stdin
   or: etm+ [?|help]    print this usage information

With the environmental variable ETMHOME set to your etm
root directory, text either piped to this script or
provided as arguments will be appended to 'inbasket.text'
in the ETMHOME directory. When this file exists, etm will
display an ⓘ character at the right end of status bar
alerting you that this file is available for import by
pressing F5.

If the text provided to this script does not begin with an
etm typechar in  -, *, % or !, then the default typechar
'{typechar}' will be used. If the provided string does not
contain any @-key options, then '{option}' will be
appended.

If the inbox typechar '!' is used then after importing,
the reminder will appear as an 'inbox' item requiring your
attention in the list for the current day in agenda view.
This may be especially useful in composing quick notes with
the assurance that you will be reminded to sort them out
later. """

    etmhome = os.environ.get("ETMHOME")
    if not etmhome:
        print("The environmental variable 'ETMHOME' is missing but required.")
        sys.exit()
    elif not os.path.isdir(etmhome):
        print(f"The environmental variable 'ETMHOME={etmhome}' is not a valid directory.")
        sys.exit()

    inbasket = os.path.join(etmhome, 'inbasket.text')


    # use stdin if it's full
    if not sys.stdin.isatty():
        input = sys.stdin.read()

    # otherwise, get the input from
    elif len(sys.argv) > 1:
        if len(sys.argv) == 2:
            input = " ".join(sys.argv[1:])
        else:
            print("The provided input should be wrapped in single quotes")
            sys.exit()
    else:
        print(help)
        sys.exit()

    input = input.strip()
    if input in ["help", "?"]:
        print(help)
        sys.exit()

    # if input does not begins with an itemtype character, prepend typechar .
    input = input if input[0] in "!*-%" else f"{typechar} {input}"
    input = input if '@' in input else f"{input} {option}"

    with open(inbasket, 'a') as fo:
        fo.write(f"{input}\n")
    print(f"appended:\n   '{input}'\nto {inbasket}")

if __name__ == "__main__":
    main()


